/***************************************************************************
 *            ClientConfig.cc
 *
 *  Mon Feb 28 22:52:32 2005
 *  Copyright  2005  Chris Wilson
 *  anjuta@qwirx.com
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <wx/file.h>

#include "Box.h"
#include "BackupDaemonConfigVerify.h"

#include "main.h"
#include "ClientConfig.h"

ClientConfig::ClientConfig(const wxString& configFileName) {
	std::string errs;
	std::auto_ptr<Configuration> config(Configuration::LoadAndVerify(
		configFileName.c_str(), &BackupDaemonConfigVerify, errs));
		
	this->mConfigFileName = configFileName;
	
	if (config.get() == 0 || !errs.empty())
	{
		wxString *exception = new wxString();
		exception->Printf(
			"Invalid configuration file:\n%s", errs.c_str());
		throw exception;
	}

	// Easier coding
	conf = new Configuration(*config);

	#define STR_PROP(name) \
		name = new StringProperty(#name, conf);
	#define STR_PROP_SUBCONF(name, subconf) \
		name = new StringProperty(#name, &( conf->GetSubConfiguration(#subconf) ));
	#define INT_PROP(name) \
		name = new IntProperty(#name, conf);
	#define BOOL_PROP(name) \
		name = new BoolProperty(#name, conf);
	ALL_PROPS
	#undef BOOL_PROP
	#undef INT_PROP
	#undef STR_PROP_SUBCONF
	#undef STR_PROP

	const Configuration &locations(conf->GetSubConfiguration("BackupLocations"));

	for(std::list<std::pair<std::string, Configuration> >::const_iterator i = 
		locations.mSubConfigurations.begin();
		i != locations.mSubConfigurations.end(); 
		i++)
	{
		std::string name = i->first;
		std::string path = i->second.GetKeyValue("Path");
		
		Location *pLoc = new Location(wxString(name.c_str()), 
			wxString(path.c_str()));

		MyExcludeList* pExcluded = new MyExcludeList(i->second);
		pLoc->SetExcludeList(pExcluded);
		
		mLocations.push_back(pLoc);
	}
}	

void ClientConfig::AddLocation(Location* newLoc) {
	mLocations.push_back(newLoc);
}

void ClientConfig::ReplaceLocation(int index, Location* newLoc) {
	mLocations[index] = newLoc;
}

void ClientConfig::RemoveLocation(int target) {
	std::vector<Location*>::iterator current = mLocations.begin();
	int i;
	for (i = 0; i < target; i++) {
		current++;
	}
	if (i == target)
		mLocations.erase(current);
}

bool ClientConfig::Save() {
	wxString filename = GetConfigFileName();
	// filename.Append(".new");
	wxTempFile file(filename);
	
	if (!file.IsOpened()) {
		return false;
	}
	
	wxString buffer;

	#define WRITE_STR_PROP(name) \
	if (const std::string * value = name->Get()) { \
		buffer.Printf(#name " = %s\n", value->c_str()); \
		file.Write(buffer); \
	}

	#define WRITE_INT_PROP_FMT(name, fmt) \
	if (const int * value = name->Get()) { \
		buffer.Printf(#name " = " fmt "\n", *value); \
		file.Write(buffer); \
	}
	
	#define WRITE_INT_PROP(name) \
	if (const int * value = name->Get()) { \
		buffer.Printf(#name " = %d\n", *value); \
		file.Write(buffer); \
	}

	#define WRITE_BOOL_PROP(name) \
	if (const bool * value = name->Get()) { \
		buffer.Printf(#name " = %s\n", *value ? "yes" : "no"); \
		file.Write(buffer); \
	}
	
	WRITE_STR_PROP(StoreHostname)
	WRITE_INT_PROP(StorePort)
	WRITE_INT_PROP_FMT(AccountNumber, "0x%x")
	WRITE_STR_PROP(KeysFile)
	WRITE_STR_PROP(CertificateFile)
	WRITE_STR_PROP(PrivateKeyFile)
	WRITE_STR_PROP(TrustedCAsFile)
	WRITE_STR_PROP(DataDirectory)
	WRITE_STR_PROP(NotifyScript)
	WRITE_INT_PROP(UpdateStoreInterval)
	WRITE_INT_PROP(MinimumFileAge)
	WRITE_INT_PROP(MaxUploadWait)
	WRITE_INT_PROP(FileTrackingSizeThreshold)
	WRITE_INT_PROP(DiffingUploadSizeThreshold)
	WRITE_INT_PROP(MaximumDiffingTime)
	WRITE_BOOL_PROP(ExtendedLogging)
	WRITE_STR_PROP(SyncAllowScript)
	WRITE_STR_PROP(CommandSocket)

	#undef WRITE_STR_PROP
	#undef WRITE_INT_PROP
	#undef WRITE_INT_PROP_FMT
	#undef WRITE_BOOL_PROP
	
	if (PidFile->Get()) {
		std::string pidbuf;
		PidFile->GetInto(pidbuf);
		buffer.Printf("Server\n{\n\tPidFile = %s\n}\n", pidbuf.c_str());
		file.Write(buffer);
	}
	
	const std::vector<Location*>& rLocations = GetLocations();
	file.Write("BackupLocations\n{\n");
	
	for (size_t i = 0; i < rLocations.size(); i++) {
		Location* pLoc = rLocations[i];
		buffer.Printf("\t%s\n\t{\n\t\tPath = %s\n", pLoc->GetName().c_str(), 
			pLoc->GetPath().c_str());
		file.Write(buffer);
		
		const std::vector<MyExcludeEntry*>& rEntries = 
			pLoc->GetExcludeList().GetEntries();
		
		for (size_t l = 0; l < rEntries.size(); l++) {
			MyExcludeEntry* pEntry = rEntries[l];
			buffer.Printf("\t\t%s\n", pEntry->ToString().c_str());
			file.Write(buffer);
		}
		
		file.Write("\t}\n");
	}
	
	file.Write("}\n");
	
	file.Commit();
	
	return true;
}
