/*
 * Main source code file for Boxi, a graphical user interface for
 * Box Backup (software developed by Ben Summers).
 * 
 * (C) Chris Wilson <boxi_main.cc@qwirx.com>, 2004-05
 * Licensed under the GNU General Public License (GPL) version 2 or later
 *
 * Please note: This product includes software developed by Ben Summers.
 * The above attribution must not be removed or modified!
 */

#include <signal.h>

#include "main.h"

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <wx/wx.h>
#include <wx/notebook.h>
#include <wx/cmdline.h>

#include "BackupFilesPanel.h"
#include "BackupLocationsPanel.h"
#include "ClientInfoPanel.h"
#include "ServerFilesPanel.h"

// GUI code starts

void AddParam(wxPanel* panel, const char* label, wxWindow* editor, 
	bool growToFit, wxSizer *sizer) 
{
	sizer->Add(new wxStaticText(panel, -1, wxString(label), 
		wxDefaultPosition), 0, wxALIGN_CENTER_VERTICAL);

	if (growToFit) {
		sizer->Add(editor, 1, wxGROW);
	} else {
		sizer->Add(editor, 1, wxALIGN_CENTER_VERTICAL);
	}
}

static wxString theGlobalConfigFileName;

class MainFrame : public wxFrame {
	public:
	MainFrame(const wxString& title, const wxPoint& pos, 
			const wxSize& size, long style = wxDEFAULT_FRAME_STYLE);
	
	void OnFileOpen			(wxCommandEvent&	event);
	void OnFileSave			(wxCommandEvent&	event);
	void OnFileQuit			(wxCommandEvent&	event);
	void OnFileDir			(wxCommandEvent&	event);
	void OnHelpAbout		(wxCommandEvent&	event);
	void OnSize				(wxSizeEvent&		event);
	
	private:
	// wxTextCtrl *theText;
	// wxStatusBar 		*theStatusBar;
	wxString    		 theConfigFileName;
	ClientConfig		*mpConfig;

	wxNotebook			*theTopNotebook;
	BackupFilesPanel	*theBackupFilesPanel;
	wxPanel				*theClientPanel;
	wxPanel				*theLocationsPanel;
	wxPanel				*theServerFilesPanel;

	DECLARE_EVENT_TABLE()
};	

MainFrame::MainFrame(const wxString& title, const wxPoint& pos,
	const wxSize& size, long style)
	: wxFrame(NULL, -1, title, pos, size, style)
{
	this->theConfigFileName = theGlobalConfigFileName;
	mpConfig = new ClientConfig(theConfigFileName);
	
	theTopNotebook = new wxNotebook(this, ID_Top_Notebook);

	theClientPanel = new ClientInfoPanel(mpConfig,
		theTopNotebook, ID_Client_Panel);
	theTopNotebook->AddPage(theClientPanel, "Configuration");

	theLocationsPanel = new BackupLocationsPanel(mpConfig,
		theTopNotebook, ID_Client_Panel);
	theTopNotebook->AddPage(theLocationsPanel, "Locations");

	theBackupFilesPanel = new BackupFilesPanel(mpConfig,
		theTopNotebook, ID_Backup_Files_Panel);
	theTopNotebook->AddPage(theBackupFilesPanel, "Backup Files");

	theServerFilesPanel = new ServerFilesPanel(mpConfig,
		theTopNotebook, ID_Backup_Files_Panel);
	theTopNotebook->AddPage(theServerFilesPanel, "Server Files");
	
	wxMenu *menuFile = new wxMenu;
	menuFile->Append(ID_File_Open,  "&Open...\tCtrl-O", "Open client config file");
	menuFile->Append(ID_File_Save,  "&Save...\tCtrl-S", "Save current config file");
	menuFile->Append(ID_File_Quit,  "E&xit\tAlt-F4", "Quit this program");

	wxMenu *menuHelp = new wxMenu;
	menuHelp->Append(ID_Help_About, "&About...\tF1", "Show About dialog");

	wxMenuBar *menuBar = new wxMenuBar;
	menuBar->Append(menuFile, "&File");
	menuBar->Append(menuHelp, "&Help");
	SetMenuBar(menuBar);
	
	// theStatusBar = CreateStatusBar(3);
}

BEGIN_EVENT_TABLE(MainFrame, wxFrame)
	EVT_MENU(ID_File_Open,  MainFrame::OnFileOpen)
	EVT_MENU(ID_File_Save,  MainFrame::OnFileSave)
	EVT_MENU(ID_File_Quit,  MainFrame::OnFileQuit)
	EVT_MENU(ID_Help_About, MainFrame::OnHelpAbout)
	EVT_SIZE(MainFrame::OnSize)
END_EVENT_TABLE()

static const
 wxChar
  *FILETYPES = _T( "Text files|*.txt|"
                   "C/C++ source files|*.cpp;*.cc;*.c|"
                   "C/C++ header files|*.hpp;*.h|"
                   "Make files|Mak*;mak*|"
                   "Java files|*java|"
                   "Hypertext markup files|*html;*htm;*HTML;*HTM|"
                   "All files|*.*"
                  );

void MainFrame::OnFileOpen(wxCommandEvent& event) {
	wxFileDialog *openFileDialog = new wxFileDialog(
		this, "Open file", "", "", FILETYPES, wxOPEN, wxDefaultPosition);

	if (openFileDialog->ShowModal() == wxID_OK) {
		// theText->LoadFile(openFileDialog->GetFilename());
		SetStatusText(openFileDialog->GetFilename(), 0);
	}
}

void MainFrame::OnFileSave(wxCommandEvent& event) {
	mpConfig->Save();
}

void MainFrame::OnFileQuit(wxCommandEvent& event) {
	Close(TRUE);
}

void MainFrame::OnFileDir(wxCommandEvent& event) {
	wxDirDialog *d = new wxDirDialog(
		this, "Choose a Directory", wxGetCwd(), 0, wxDefaultPosition);

	if (d->ShowModal() == wxID_OK)
		wxSetWorkingDirectory(d->GetPath());
}

void MainFrame::OnHelpAbout(wxCommandEvent& event) {
	wxMessageBox("Boxi " BOXI_VERSION " by Chris Wilson\n"
		"This product includes software developed by Ben Summers.",
		"About Boxi", wxOK | wxICON_INFORMATION, this);
}

void MainFrame::OnSize(wxSizeEvent& event) 
{
	wxString theSizeText;
	wxSize theNewSize = event.GetSize();
	theSizeText.Printf("%d x %d", theNewSize.GetWidth(), 
		theNewSize.GetHeight());
	// theText->SetValue(theSizeText);
	wxFrame::OnSize(event);
}

static wxCmdLineEntryDesc theCmdLineParams[] = 
{
	{ wxCMD_LINE_PARAM, "", "", "<bbackupd-config-file>",
		wxCMD_LINE_VAL_STRING, wxCMD_LINE_OPTION_MANDATORY },
	{ wxCMD_LINE_NONE },
};

class BoxiApp : public wxApp
{
  	public:
    virtual bool OnInit();
};

IMPLEMENT_APP(BoxiApp)

void sigpipe_handler(int signum) {
	signal(SIGPIPE, sigpipe_handler);
}

bool BoxiApp::OnInit()
{
	wxCmdLineParser cmdParser(theCmdLineParams, argc, argv);
	int result = cmdParser.Parse();
	
	if (result != 0)
		return FALSE;
	
	theGlobalConfigFileName = cmdParser.GetParam();

  	wxFrame *frame = new MainFrame (
		"Boxi " BOXI_VERSION,
    	wxPoint(50, 50), wxSize(600, 500)
	);
	
  	frame->Show(TRUE);
	
	// Initialise global SSL system configuration/state
	SSLLib::Initialise();
	
	signal(SIGPIPE, sigpipe_handler);
	
  	return TRUE;
}
